@@ -48,14 +48,13 @@ class Property(object):
     Creates a new property which in conjunction with GObject subclass will
     create a property proxy:
 
-    >>> class MyObject(GObject.GObject):
-    >>> ... prop = GObject.Property(type=str)
+         class MyObject(GObject.GObject):
+         ... prop = GObject.Property(type=str)
 
-    >>> obj = MyObject()
-    >>> obj.prop = 'value'
+         obj = MyObject()
+         obj.prop = 'value'
 
-    >>> obj.prop
-    'value'
+         obj.prop  # now is 'value'
 
     The API is similar to the builtin property:
 
@@ -261,7 +260,8 @@ class Property(object):
               issubclass(type_, (_gobject.GObject,
                                  _gobject.GEnum,
                                  _gobject.GFlags,
-                                 _gobject.GBoxed))):
+                                 _gobject.GBoxed,
+                                 _gobject.GInterface))):
             return type_.__gtype__
         elif type_ in (TYPE_NONE, TYPE_INTERFACE, TYPE_CHAR, TYPE_UCHAR,
                        TYPE_INT, TYPE_UINT, TYPE_BOOLEAN, TYPE_LONG,
